package com.qmth.eds.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import com.qmth.eds.service.CloudMarkingScoreForeignService;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 学校表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2022-05-14
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/cloud_marking_score_foreign")
public class CloudMarkingScoreForeignController {

    @Resource
    private CloudMarkingScoreForeignService cloudMarkingScoreForeignService;

    /**
     * 查询
     *
     * @param semesterId 学期ID
     * @param examTypeId 考试类型ID
     * @param examId     云阅卷考试ID
     * @param pageNumber 分页参数
     * @param pageSize   分页参数
     */
    @ApiOperation(value = "查询")
    @PostMapping("/page")
    public Result page(@RequestParam(value = "semesterId", required = false) Long semesterId,
                       @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                       @RequestParam(value = "examId", required = false) String examId,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(cloudMarkingScoreForeignService.pageData(semesterId, examTypeId, examId, pageNumber, pageSize));
    }

    /**
     * 查询
     *
     * @param semesterId 学期ID
     * @param examTypeId 考试类型ID
     * @param examId     云阅卷考试ID
     */
    @ApiOperation(value = "下载")
    @PostMapping("/download_score")
    public void downloadScore(@RequestParam(value = "semesterId", required = false) Long semesterId,
                              @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                              @RequestParam(value = "examId", required = false) String examId,
                              HttpServletResponse response) {
        cloudMarkingScoreForeignService.downloadScore(semesterId, examTypeId, examId, response);
    }
}
