package com.qmth.eds.common.entity;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.eds.common.enums.TaskTypeEnum;
import com.qmth.eds.core.base.BaseEntity;
import com.qmth.eds.common.enums.TaskResultEnum;
import com.qmth.eds.common.enums.TaskStatusEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;

/**
 * <p>
 * 同步任务表
 * </p>
 */
@ApiModel(value = "TBSyncTask对象", description = "同步任务表")
public class TBSyncTask extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校ID")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("school_id")
    private Long schoolId;

    @ApiModelProperty(value = "学期ID")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(value = "semester_id")
    private Long semesterId;

    @ApiModelProperty(value = "考试类型ID")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("exam_type_id")
    private Long examTypeId;

    @ApiModelProperty(value = "任务类型")
    @TableField(value = "type")
    private TaskTypeEnum type;

    @ApiModelProperty(value = "执行时间")
    @TableField(value = "sync_time")
    private Long syncTime;

    @ApiModelProperty(value = "任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成")
    @TableField(value = "status")
    private TaskStatusEnum status;

    @ApiModelProperty(value = "数据结果，SUCCESS：成功，ERROR：失败")
    @TableField(value = "result")
    private TaskResultEnum result;

    @ApiModelProperty(value = "备注")
    @TableField(value = "remark")
    private String remark;

    @ApiModelProperty(value = "错误原因")
    @TableField(value = "error_message", updateStrategy = FieldStrategy.IGNORED)
    private String errorMessage;

    public TBSyncTask() {
    }

    public TBSyncTask(Long schoolId, Long semesterId, Long examTypeId, TaskTypeEnum type, String remark, Long createId) {
        this.schoolId = schoolId;
        this.semesterId = semesterId;
        this.examTypeId = examTypeId;
        this.type = type;
        this.remark = remark;
        this.status = TaskStatusEnum.INIT;
        this.syncTime = System.currentTimeMillis();
        this.setCreateId(createId);
        this.setCreateTime(System.currentTimeMillis());
    }

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public Long getExamTypeId() {
        return examTypeId;
    }

    public void setExamTypeId(Long examTypeId) {
        this.examTypeId = examTypeId;
    }

    public TaskTypeEnum getType() {
        return type;
    }

    public void setType(TaskTypeEnum type) {
        this.type = type;
    }

    public Long getSyncTime() {
        return syncTime;
    }

    public void setSyncTime(Long syncTime) {
        this.syncTime = syncTime;
    }

    public TaskStatusEnum getStatus() {
        return status;
    }

    public void setStatus(TaskStatusEnum status) {
        this.status = status;
    }

    public TaskResultEnum getResult() {
        return result;
    }

    public void setResult(TaskResultEnum result) {
        this.result = result;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }
}
