package com.qmth.eds.common.tools;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.HttpKit;
import com.qmth.eds.core.config.DictionaryConfig;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 云阅卷接口工具类
 */
@Service
public class CloudMarkingUtil {

    private static final String SUCCESS = "success";

    private static final String EXAMID = "examId";
    private static final String EXAM_NUMBER = "examNumber";
    private static final String STUDENT_CODE = "studentCode";
    private static final String COLLEGE = "college";
    private static final String CLASS_NAME = "className";
    private static final String TEACHER = "teacher";
    private static final String SUBJECT_CODE = "subjectCode";


    @Resource
    DictionaryConfig dictionaryConfig;

    /**
     * 考试成绩考生查询接口
     *
     * @param pageNo   分页参数
     * @param pageSize 分页参数
     */
    public List<JSONObject> getExam(int pageNo, int pageSize) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("pageNumber", String.valueOf(pageNo));
        paramMap.put("pageSize", String.valueOf(pageSize));

        String yyjRootUrl = dictionaryConfig.cloudMarkingDomain().getRootUrl();
        String examUrl = dictionaryConfig.cloudMarkingDomain().getExamUrl();

        if (StringUtils.isAnyBlank(yyjRootUrl, examUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试查询接口url未设置");
        }

        Map<String, String> header = createHeader(examUrl);
        //请求
        String examObject;
        try {
            examObject = HttpKit.sendPost(yyjRootUrl + examUrl, paramMap, header);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询考试接口调用异常");
        }
        return JSONObject.parseArray(JSONObject.toJSON(examObject).toString(), JSONObject.class);
    }

    /**
     * 考试成绩考生数量查询接口
     *
     * @param mapSource 参数
     */
    public int getStudentCount(Map<String, String> mapSource) {
        Map<String, String> map = paramMap(mapSource);

        String yyjRootUrl = dictionaryConfig.cloudMarkingDomain().getRootUrl();
        String studentCountUrl = dictionaryConfig.cloudMarkingDomain().getStudentCountUrl();

        if (StringUtils.isAnyBlank(yyjRootUrl, studentCountUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试成绩考生数量查询接口url未设置");
        }

        //header
        Map<String, String> headerMap = createHeader(studentCountUrl);
        //请求
        String res = HttpKit.sendPost(yyjRootUrl + studentCountUrl, map, headerMap);
        if (StringUtils.isNotBlank(res)) {
            JSONObject object = JSONObject.parseObject(res);
            return Integer.parseInt(object.get("totalCount").toString());
        }
        return 0;
    }

    /**
     * 考试成绩考生查询接口
     *
     * @param map      参数
     * @param pageNo   分页参数
     * @param pageSize 分页参数
     */
    public List<JSONObject> getStudentScore(Map<String, String> map, int pageNo, int pageSize) {
        Map<String, String> paramMap = paramMap(map);

        paramMap.put("pageNumber", String.valueOf(pageNo));
        paramMap.put("pageSize", String.valueOf(pageSize));

        String yyjRootUrl = dictionaryConfig.cloudMarkingDomain().getRootUrl();
        String studentScoreUrl = dictionaryConfig.cloudMarkingDomain().getStudentScoreUrl();

        if (StringUtils.isAnyBlank(yyjRootUrl, studentScoreUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试成绩查询接口url未设置");
        }

        Map<String, String> header = createHeader(studentScoreUrl);
        //请求
        String studentObject = HttpKit.sendPost(yyjRootUrl + studentScoreUrl, paramMap, header);
        return JSONObject.parseArray(JSONObject.toJSON(studentObject).toString(), JSONObject.class);
    }

    /**
     * 云阅卷接口请求header
     *
     * @param url url
     */
    private Map<String, String> createHeader(String url) {
        long time = System.currentTimeMillis();
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("time", String.valueOf(time));
        String accessKey = dictionaryConfig.cloudMarkingDomain().getAccessKey();
        String accessSecret = dictionaryConfig.cloudMarkingDomain().getAccessSecret();
        String signature = SignatureEntity.build(SignatureType.SECRET, "POST", url, time, accessKey, accessSecret);
        headerMap.put("authorization", signature);

        return headerMap;
    }

    private Map<String, String> paramMap(Map<String, String> mapSource) {
        Map<String, String> map = new HashMap<>();
        Integer examId = Integer.valueOf(mapSource.get(EXAMID));
        if (Objects.nonNull(examId)) {
            map.put(EXAMID, String.valueOf(examId));
        }
        String examCode = mapSource.get("examCode");
        if (StringUtils.isNotBlank(examCode)) {
            map.put("examCode", examCode);
        }
        String examNumber = mapSource.get(EXAM_NUMBER);
        if (StringUtils.isNotBlank(examNumber)) {
            map.put(EXAM_NUMBER, examNumber);
        }
        String studentCode = mapSource.get(STUDENT_CODE);
        if (StringUtils.isNotBlank(studentCode)) {
            map.put(STUDENT_CODE, studentCode);
        }
        String subjectCode = mapSource.get(SUBJECT_CODE);
        if (StringUtils.isNotBlank(subjectCode)) {
            map.put(SUBJECT_CODE, subjectCode);
        }
        String college = mapSource.get(COLLEGE);
        if (StringUtils.isNotBlank(college)) {
            map.put(COLLEGE, college);
        }
        String className = mapSource.get(CLASS_NAME);
        if (StringUtils.isNotBlank(className)) {
            map.put(CLASS_NAME, className);
        }
        String teacher = mapSource.get(TEACHER);
        if (StringUtils.isNotBlank(teacher)) {
            map.put(TEACHER, teacher);
        }
        return map;
    }
}
