package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.ExamSyncTotalDownloadDto;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.ExamDownloadRecord;
import com.qmth.eds.common.entity.ExamSyncTotal;
import com.qmth.eds.common.entity.SysUser;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.mapper.ExamSyncTotalMapper;
import com.qmth.eds.service.ExamDownloadRecordService;
import com.qmth.eds.service.ExamSyncTotalService;
import com.qmth.eds.service.SysUserService;
import com.qmth.eds.common.util.FileUtil;
import com.qmth.eds.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.List;

@Service
public class ExamSyncTotalServiceImpl extends ServiceImpl<ExamSyncTotalMapper, ExamSyncTotal> implements ExamSyncTotalService {

    @Resource
    private SysUserService sysUserService;

    @Resource
    private ExamDownloadRecordService examDownloadRecordService;

    @Override
    public IPage<ExamSyncTotal> list(Long syncDate, Integer pageNumber, Integer pageSize) {
        QueryWrapper<ExamSyncTotal> queryWrapper = new QueryWrapper<>();
        Page<ExamSyncTotal> page = new Page<>(pageNumber, pageSize);
        if (syncDate != null) {
            queryWrapper.lambda().eq(ExamSyncTotal::getSyncDate, syncDate);
        }
        queryWrapper.lambda().orderByDesc(ExamSyncTotal::getCreateTime);
        return this.baseMapper.selectPage(page, queryWrapper);
    }

    @Override
    public void download(Long id, Boolean writeLog, HttpServletResponse response) {
        ExamSyncTotal examSyncTotal = this.getById(id);
        String fileName = examSyncTotal.getFileName();
        String filePath = examSyncTotal.getFilePath();
        File file = new File(filePath);
        if (!file.exists()) {
            throw ExceptionResultEnum.ERROR.exception("文件不存在");
        }

        if (writeLog) {
            // 记录日志
            Long userId = ServletUtil.getRequestUserId();
            SysUser sysUser = sysUserService.getById(userId);

            ExamDownloadRecord examDownloadRecord = new ExamDownloadRecord();
            examDownloadRecord.setId(SystemConstant.getDbUuid());
            examDownloadRecord.setOperateTime(System.currentTimeMillis());
            examDownloadRecord.setUserId(userId);
            examDownloadRecord.setUserName(sysUser.getRealName());
            examDownloadRecord.setSyncTotalId(id);
            examDownloadRecord.setUsed(false);
            examDownloadRecord.setCreateId(userId);
            examDownloadRecord.setCreateTime(System.currentTimeMillis());
            examDownloadRecordService.save(examDownloadRecord);
        }

        // 下载
        FileUtil.outputFile(response, file, fileName);
    }

    @Override
    public void updateDownloadStatusBySchoolIdAndSemesterIdAndExamTypeId(Long schoolId, Long semesterId, Long examTypeId) {
        UpdateWrapper<ExamSyncTotal> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamSyncTotal::getDownloadStatus, false)
                .eq(ExamSyncTotal::getSchoolId, schoolId)
                .eq(ExamSyncTotal::getSemesterId, semesterId)
                .eq(ExamSyncTotal::getExamTypeId, examTypeId);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamSyncTotalDownloadDto> listDownloadRecordInfos(Long schoolId, Long semesterId, Long examTypeId) {
        return this.baseMapper.listDownloadRecordInfos(schoolId, semesterId, examTypeId);
    }
}
