package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.result.SyncTaskListResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.ExamScheduleTask;
import com.qmth.eds.common.entity.TBSyncTask;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.enums.TaskResultEnum;
import com.qmth.eds.common.enums.TaskStatusEnum;
import com.qmth.eds.common.enums.TaskTypeEnum;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.TBSyncTaskMapper;
import com.qmth.eds.service.TBSyncTaskService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

/**
 * 异步任务日志
 */
@Service
public class TBSyncTaskServiceImpl extends ServiceImpl<TBSyncTaskMapper, TBSyncTask> implements TBSyncTaskService {

    @Transactional
    @Override
    public TBSyncTask saveTask(ExamScheduleTask examScheduleTask, TaskTypeEnum type, boolean isAuto) {
        Long userId = isAuto ? 0L : ServletUtil.getRequestUserId();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, examScheduleTask.getSchoolId())
                .eq(TBSyncTask::getSemesterId, examScheduleTask.getSemesterId())
                .eq(TBSyncTask::getExamTypeId, examScheduleTask.getExamTypeId())
                .eq(TBSyncTask::getType, type);
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            this.removeById(tbSyncTask.getId());
        }
        tbSyncTask = new TBSyncTask(examScheduleTask.getSchoolId(), examScheduleTask.getSemesterId(), examScheduleTask.getExamTypeId(), type, null, userId);
        tbSyncTask.setId(SystemConstant.getDbUuid());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Override
    public TBSyncTask saveTask(Long semesterId, Long examTypeId, TaskTypeEnum type, String remark) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Long userId = ServletUtil.getRequestUserId();
        QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBSyncTask::getSchoolId, schoolId)
                .eq(TBSyncTask::getSemesterId, semesterId)
                .eq(TBSyncTask::getExamTypeId, examTypeId)
                .eq(TBSyncTask::getType, type);
        TBSyncTask tbSyncTask = this.getOne(queryWrapper);
        if (tbSyncTask != null) {
            if (!TaskStatusEnum.FINISH.equals(tbSyncTask.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("有任务未结束");
            }
            this.removeById(tbSyncTask.getId());
        }
        tbSyncTask = new TBSyncTask(schoolId, semesterId, examTypeId, type, remark, userId);
        tbSyncTask.setId(SystemConstant.getDbUuid());
        this.save(tbSyncTask);
        return tbSyncTask;
    }

    @Override
    public void updateStatusAndResultById(Long id, TaskStatusEnum status, TaskResultEnum result, String errorMessage) {
        UpdateWrapper<TBSyncTask> tbSyncTaskUpdateWrapper = new UpdateWrapper<>();
        tbSyncTaskUpdateWrapper.lambda().set(TBSyncTask::getStatus, status)
                .set(TBSyncTask::getResult, result)
                .set(TBSyncTask::getErrorMessage, errorMessage)
                .eq(TBSyncTask::getId, id);
        this.update(tbSyncTaskUpdateWrapper);
    }

    @Override
    public IPage<SyncTaskListResult> query(Page<SyncTaskListResult> iPage, TaskStatusEnum status, TaskTypeEnum type, TaskResultEnum result) {
        Long userId = ServletUtil.getRequestUserId();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.query(iPage, schoolId, Objects.nonNull(status) ? status.name() : null, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(result) ? result.name() : null, userId);
    }
}
